@@ -1,9 +1,10 @@
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: t; c-basic-offset: 2 -*- */
+/* vi: set ts=2 sw=2: */
 /* error.h
 
    GJITEN : A GTK+/GNOME BASED JAPANESE DICTIONARY
   
-   Copyright (C) 1999 - 2003 Botond Botyanszki <boti@rocketmail.com>
+   Copyright (C) 1999 - 2005 Botond Botyanszki <boti@rocketmail.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published  by
@@ -24,7 +25,7 @@
 #define __ERROR_H__
 
 #ifdef DEBUG_GJITEN
-#define GJITEN_DEBUG(...) g_print(__VA_ARGS__)
+#define GJITEN_DEBUG(...) fprintf(stderr, __VA_ARGS__)
 #else 
 #define GJITEN_DEBUG(...)
 #endif
