@@ -46,7 +46,6 @@
 #include "radical-convtable.h"
 
 /*====== Prototypes========================================================*/
-int get_radical_index(gunichar radical);
 void get_rad_of_kanji(gunichar kanji);
 
 static void kanjidic_close();
@@ -58,18 +57,7 @@ gchar *radkfile = NULL;
 guint32 radkfile_size;
 extern guint32 srchpos;
 
-struct knode {
-  gunichar kanji;
-  struct knode *next;
-} *klinklist, *tmpklinklist;
-
-struct radical_info {
-  gunichar radical;
-  int strokes;
-} radicals[300];
-
-gunichar *radical_kanji = NULL;
-int total_radicals, radical_kanji_start[300], radical_kanji_count[300];
+GList *klinklist = NULL, *tmpklinklist = NULL;
 
 KanjiDic *kanjiDic;
 extern gchar *strginfo[];
@@ -135,12 +123,13 @@ void do_kdicline(gchar *kstr) {
   
   g_unichar_to_utf8(g_utf8_get_char(kstr), &kdic_line[KANJI * KBUFSIZE] ); //KANJI
   get_rad_of_kanji(g_utf8_get_char(kdic_line + KANJI * KBUFSIZE)); //RADICAL
-  
+
   get_word(kdic_line + JIS * KBUFSIZE, kstr, KBUFSIZE, 3);
   pos = 7;
-  //printf("%s\n",kstr);
+
   while (pos != 0) {
     pos = get_word(tmpstr, kstr, sizeof(tmpstr), pos);
+
     if ((tmpstr[0] >> 7)) {       // jap char   //FIXME
      	if (strlen(kdic_line + READING * KBUFSIZE) != 0) {
 				strncat(kdic_line + READING * KBUFSIZE, ", ", KBUFSIZE - strlen(kdic_line + READING * KBUFSIZE) - 1);
@@ -248,7 +237,7 @@ void do_kdicline(gchar *kstr) {
     
 				case 'M':
 					if (tmpstr[1] == 'N') strncpy(kdic_line + MNINDEX * KBUFSIZE, tmpstr + 2, KBUFSIZE);
-					else if (tmpstr[1] == 'P') strncpy(kdic_line + MPINDEX, tmpstr + 2, KBUFSIZE);
+					else if (tmpstr[1] == 'P') strncpy(kdic_line + MPINDEX * KBUFSIZE, tmpstr + 2, KBUFSIZE);
 					break;
       
 				case 'E':
@@ -297,7 +286,6 @@ void print_kanjinfo(gunichar kanji) {
   for (i = 0; i < 6; i++) kanjistr[i] = 0;
   g_unichar_to_utf8(kanji, kanjistr);
 
-  
   gtk_text_buffer_set_text(GTK_TEXT_BUFFER(kanjiDic->text_kanjinfo_buffer), "", 0);
   gtk_text_buffer_get_start_iter(kanjiDic->text_kanjinfo_buffer, &kanjiDic->kinfo_iter);
 
@@ -328,119 +316,33 @@ void print_kanjinfo(gunichar kanji) {
 }
   
 
-/*
-//this is only for debugging 
-void klist_print(struct knode *list) { 
-  gchar kanji[6];
-  int i;
-  struct knode *nodeptr;
-  
-  i = 0;
-  printf("Elements in list:\n");
-  nodeptr = list;
-  while (nodeptr != NULL) {
-    g_unichar_to_utf8(*(nodeptr->kanji), kanji);
-    printf("%s ", kanji);
-    nodeptr = nodeptr->next;
-    i++;
-  }
-  printf("\n----TOTAL: %d-----\n", i);
-}
-*/
-
-
-//
-// FIXME: convert this to g_slist
-//
-void knode_add(struct knode **llist, gunichar kanji) { 
-/* adds a node to the linked list with a kanji. */
-
-  struct knode *currnode = NULL, *prevnode = NULL;
-  
-  currnode = *llist;
-  while (currnode != NULL) { 
-    if ((currnode->kanji) == kanji) return;
-    prevnode = currnode;
-    currnode = prevnode->next;
-  }
-  currnode = g_malloc(sizeof(struct knode));
-  if (currnode == NULL) gjiten_abort_with_msg("Malloc failed (in knode_add).\n");
-  if (*llist == NULL) *llist = currnode;
-  else prevnode->next = currnode;
-  currnode->kanji = kanji;
-  currnode->next = NULL;
-}
-
-void klist_free(struct knode **llist) { // frees up a whole llist 
-  struct knode *ptr, *nextptr;
-
-  ptr = *llist;
-  if (ptr == NULL) return;
-  while (ptr != NULL) {
-    nextptr = ptr->next;
-    g_free(ptr);
-    ptr = nextptr;
-  }
-  *llist = NULL;
-}
-
-void knode_remove(struct knode **llist, struct knode *node2rem)  {
-  struct knode *currnode, *prevnode;
-
-  currnode = *llist;
-  prevnode = NULL;
-  
-  while ((currnode != node2rem) && (currnode != NULL)) {
-    prevnode = currnode;
-    currnode = currnode->next;
-  }
-  if (currnode == NULL) {
-    g_warning("couldn't remove knode at %d\n", (int)node2rem);
-    return;
-  }
-  if (prevnode != NULL) prevnode->next = currnode->next;
-  else *llist = currnode->next;
-  //  printf("removing: %s at %d\n",currnode->kanji,currnode);
-  g_free(node2rem);
-}
-
-// compares two lists and combines the matching kanji into one
-void klists_merge(struct knode **llist1, struct knode **llist2) {
-  struct knode *ptr1, *ptr2, *nextptr;
+/* compares two lists and combines the matching kanji into one */
+void klists_merge(void) {
+  GList *ptr1, *ptr2, *nextptr;
   int found;
 
-  ptr1 = *llist1;
-
+  ptr1 = klinklist;
   while (ptr1 != NULL) {
-    nextptr = ptr1->next;
+    nextptr = g_list_next(ptr1);
     found = FALSE;
-    ptr2 = *llist2;
+    ptr2 = tmpklinklist;
     while (ptr2 != NULL) {
-      if (ptr1->kanji == ptr2->kanji) {
-	found = TRUE;
-	break;
+      if ((gunichar) ptr1->data == (gunichar) ptr2->data) {
+				found = TRUE;
+				break;
       }
-      ptr2 = ptr2->next;
+      ptr2 = g_list_next(ptr2);
     }
-    if (found == FALSE) knode_remove(llist1, ptr1);
+    if (found == FALSE) {
+			klinklist = g_list_remove(klinklist, ptr1->data);
+		}
     ptr1 = nextptr;
   }
-  klist_free(llist2); 
+  g_list_free(tmpklinklist);
+	tmpklinklist = NULL;
 }
 
-int count_kanji(struct knode *list) { //count the nodes of llist
-  int kanjnum = 0;
-  struct knode *nodeptr;
-  
-  nodeptr = list;
-  while (nodeptr != NULL) {
-    kanjnum++;
-    nodeptr = nodeptr->next;
-  }
-  return(kanjnum);
-}
-
-void findk_by_key(gchar *srchkey, struct knode **llist)  {
+void findk_by_key(gchar *srchkey, GList **list)  {
   gint srch_resp, roff, rlen;
   gchar repstr[1024];
   guint32 respos, oldrespos; 
@@ -451,7 +353,7 @@ void findk_by_key(gchar *srchkey, struct knode **llist)  {
   //printf("F: Returning:srch_resp:%d\n respos:%ld\n roff:%d rlen:%d\n repstr:%s\n", srch_resp,respos,roff,rlen,repstr); 
   if (srch_resp != SRCH_OK) return; // (FALSE);
   oldrespos = srchpos = respos;
-  knode_add(llist, g_utf8_get_char(repstr));
+  *list = g_list_prepend(*list, (gpointer) g_utf8_get_char(repstr));
   
   while (roff != 0) {
     oldrespos = respos;
@@ -461,11 +363,11 @@ void findk_by_key(gchar *srchkey, struct knode **llist)  {
     //printf("srch_resp:%d\n respos:%ld\n roff:%d rlen:%d\n repstr:%s\n",srch_resp,respos,roff,rlen,repstr);
     if (srch_resp != SRCH_OK) break;
     if (oldrespos == respos) continue;
-    knode_add(llist, g_utf8_get_char(repstr));
+    *list = g_list_prepend(*list, (gpointer) g_utf8_get_char(repstr));
   }
 }
 
-void findk_by_stroke(int stroke, int plusmin, struct knode **llist) {
+void findk_by_stroke(int stroke, int plusmin, GList **list) {
   static char srchkey[10];
   int i, lowerlim, upperlim;
  
@@ -476,62 +378,74 @@ void findk_by_stroke(int stroke, int plusmin, struct knode **llist) {
  
   for (i = lowerlim; i <= upperlim ; i++) {
     snprintf(srchkey, 10, " S%d ", i);
-    findk_by_key(srchkey, llist);
+    findk_by_key(srchkey, list);
   }
 }
 
 void findk_by_radical(gchar *radstrg) { 
-  int i, rad_index, kanji_index, radnum;
+  gint i, radnum;
+	RadInfo *rad_info;
+	GList *kanji_info_list;
+	gchar *tmprad;
   gchar *radstr_ptr;
 
   if (g_utf8_strlen(radstrg, -1) == 0) return;
-  /*
-  if (g_utf8_strlen(radstrg) <= 1) { 
-    sprintf(tmpmsg,_("I don't seem to recognize this radical: '%s' !!!\n"),radstrg);
-    gnome_warning_dialog_parented(tmpmsg,GTK_WINDOW(kanjiDic->window));
-    return;
-  }
-  */
 
-  rad_index = get_radical_index(g_utf8_get_char(radstrg));
-  if (rad_index < 0) {
+	radstr_ptr = radstrg;
+	rad_info = g_hash_table_lookup(kanjiDic->rad_info_hash, (gpointer) g_utf8_get_char(radstr_ptr));
+  if (rad_info == NULL) {
     gjiten_print_error(_("Invalid radical!\n"));
     return;
   }
-  for (kanji_index = radical_kanji_start[rad_index]; 
-       kanji_index < radical_kanji_start[rad_index] + radical_kanji_count[rad_index]; 
-       kanji_index++) {
-    knode_add(&klinklist, radical_kanji[kanji_index]);
+
+  for (kanji_info_list = rad_info->kanji_info_list;
+			 kanji_info_list != NULL;
+			 kanji_info_list = g_list_next(kanji_info_list)) {
+    klinklist = g_list_prepend(klinklist, (gpointer) ((KanjiInfo *) kanji_info_list->data)->kanji);
   }
+
   // if we have more than 1 radical 
   radnum = g_utf8_strlen(radstrg, -1); 
   if (radnum > 1) {
-    radstr_ptr = radstrg;
     for (i = 1; i <= radnum; i++) {
-      rad_index = get_radical_index(g_utf8_get_char(radstr_ptr));
-      if (rad_index < 0) {
-				gjiten_print_error(_("I don't seem to recognize this radical: '%s' !!!\n"),
-													 g_strndup(radstr_ptr, sizeof(gunichar)));
+      rad_info = g_hash_table_lookup(kanjiDic->rad_info_hash, (gpointer) g_utf8_get_char(radstr_ptr));
+      if (rad_info == NULL) {
+				tmprad = g_strndup(radstr_ptr, sizeof(gunichar));
+				gjiten_print_error(_("I don't seem to recognize this radical: '%s' !!!\n"), tmprad);
+				g_free(tmprad);
 				return;
       }
-      for (kanji_index = radical_kanji_start[rad_index]; 
-					 kanji_index < radical_kanji_start[rad_index] + radical_kanji_count[rad_index]; 
-					 kanji_index++) {
-				knode_add(&tmpklinklist, radical_kanji[kanji_index]);
+			for (kanji_info_list = rad_info->kanji_info_list;
+					 kanji_info_list != NULL;
+					 kanji_info_list = g_list_next(kanji_info_list)) {
+				tmpklinklist = g_list_prepend(tmpklinklist, (gpointer) ((KanjiInfo *) kanji_info_list->data)->kanji);
       }
-      klists_merge(&klinklist, &tmpklinklist);
+      klists_merge();
       radstr_ptr = g_utf8_next_char(radstr_ptr);
     }
   }
 }
 
+void set_radical_button_sensitive(gpointer radical, RadInfo *rad_info, gpointer user_data) {
+	GtkWidget *rad_button = g_hash_table_lookup(kanjiDic->rad_button_hash, radical);
+	if (GTK_IS_WIDGET(rad_button) && (GTK_WIDGET_SENSITIVE(rad_button) != TRUE)) {
+		gtk_widget_set_sensitive(rad_button, TRUE);
+	}
+}
+
+void set_radical_button_unsensitive(gunichar radical, GtkWidget *rad_button, gboolean sensitive) {
+	if (GTK_IS_WIDGET(rad_button) && (GTK_WIDGET_SENSITIVE(rad_button) != sensitive)) {
+		gtk_widget_set_sensitive(rad_button, sensitive);
+	}
+}
+
 
 void on_kanji_search() {
   static gchar *kentry, *radentry;
   int found;
-  int i, j;
+  int i;
   int stroke, plus_min;
-  struct knode *node_ptr;
+  GList *node_ptr;
   gchar kappbarmsg[100];
   int push;
   GtkWidget *kanji_result_button;
@@ -539,6 +453,12 @@ void on_kanji_search() {
   gchar kanji_result_labelstr[100];
   GtkWidget *kanji_result_label;
 	GtkTextChildAnchor *kanji_results_anchor;
+	gint result_num = 0;
+	GList *rad_info_list = NULL;
+	GList *kanji_list = NULL;
+	GList *kanji_list_ptr = NULL;
+	GHashTable *rad_info_hash = NULL;
+	KanjiInfo *kanji_info;
 
   gnome_appbar_set_status(GNOME_APPBAR(kanjiDic->appbar_kanji),_("Searching..."));
   kappbarmsg[0] = 0;
@@ -547,25 +467,27 @@ void on_kanji_search() {
   gtk_text_buffer_get_start_iter(kanjiDic->kanji_results_buffer, &kanjiDic->kanji_results_iter);
 
   push = TRUE;
-  if (kentry != NULL) //Check if we need to save the key entry in the history
+  if (kentry != NULL) { //Check if we need to save the key entry in the history
     if (strcmp(kentry, gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(kanjiDic->combo_entry_key)->entry))) == 0) {  
       push = FALSE;
       g_free(kentry);
     }
+	}
   kentry = g_strdup(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(kanjiDic->combo_entry_key)->entry)));
-  //printf("KEY:%s\n", kentry);
-  if (kentry != NULL)
+  if (kentry != NULL) {
     if ((strlen(kentry) > 0) && (push == TRUE) ) {
       kanjiDic->combo_entry_key_glist = g_list_prepend(kanjiDic->combo_entry_key_glist, kentry);
       gtk_combo_set_popdown_strings(GTK_COMBO(kanjiDic->combo_entry_key),kanjiDic->combo_entry_key_glist);
     }
+	}
   push = TRUE;
-  if (radentry != NULL) //Check if we need to save the radical entry in the history
+  if (radentry != NULL) { //Check if we need to save the radical entry in the history
     if (strcmp(radentry, gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(kanjiDic->combo_entry_radical)->entry))) == 0) {  
-	push = FALSE;
-	g_free(radentry);
-      }
-  
+			push = FALSE;
+			g_free(radentry);
+		}
+  }
+
   radentry = g_strdup(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(kanjiDic->combo_entry_radical)->entry)));
   if (radentry) {
     if ((strlen(radentry) > 0) && push) {
@@ -578,11 +500,15 @@ void on_kanji_search() {
   stroke = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(kanjiDic->spinb_strokenum));
   plus_min = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(kanjiDic->spinb_plusmin));
 
-  if (klinklist != NULL) klist_free(&klinklist);
+  if (klinklist != NULL) g_list_free(klinklist);
   tmpklinklist = NULL;
   klinklist = NULL;
   found = TRUE;
 
+	if (GTK_IS_WIDGET(kanjiDic->window_radicals)) {
+		g_hash_table_foreach(kanjiDic->rad_button_hash, (GHFunc) set_radical_button_unsensitive, (gpointer) TRUE);
+	}
+
   //FIND BY RADICAL
   if ((GTK_TOGGLE_BUTTON(kanjiDic->checkb_radical)->active) && (g_utf8_strlen(radentry, -1) > 0)) {
     findk_by_radical(radentry); 
@@ -591,6 +517,7 @@ void on_kanji_search() {
       return;
     }
   }
+
   //FIND BY STROKE
   if (GTK_TOGGLE_BUTTON(kanjiDic->checkb_stroke)->active) {
     if ((stroke < 1) || (stroke > 30)) {
@@ -600,7 +527,7 @@ void on_kanji_search() {
     }
     if (klinklist == NULL) {
       findk_by_stroke(stroke, plus_min, &klinklist);  // this should! give results
-      if (klinklist == NULL) {
+      if (klinklist == NULL ) {
 				gnome_appbar_set_status(GNOME_APPBAR(kanjiDic->appbar_kanji),
 				_("Stroke search didn't find any match :-O "));
 				return;
@@ -608,7 +535,7 @@ void on_kanji_search() {
     }
     else {
       findk_by_stroke(stroke, plus_min, &tmpklinklist);
-      klists_merge(&klinklist,&tmpklinklist); 
+      klists_merge(); 
       if (klinklist == NULL) {  
 				found = FALSE; 
 				gnome_appbar_set_status(GNOME_APPBAR(kanjiDic->appbar_kanji),
@@ -617,12 +544,13 @@ void on_kanji_search() {
       }
     }
   }
+
   //FIND BY KEY
   if ((found) && (GTK_TOGGLE_BUTTON(kanjiDic->checkb_ksearch)->active) && (strlen(kentry) >= 1)) {
     if (klinklist == NULL) findk_by_key(kentry, &klinklist);
     else {
       findk_by_key(kentry, &tmpklinklist);
-      klists_merge(&klinklist, &tmpklinklist);
+      klists_merge();
     }
     if (klinklist == NULL) {    
       gnome_appbar_set_status(GNOME_APPBAR(kanjiDic->appbar_kanji), _("No Matches found!"));
@@ -630,22 +558,23 @@ void on_kanji_search() {
     }
   }
   
-  snprintf(kappbarmsg, 100, _("Kanji found: %d"), count_kanji(klinklist));
+	result_num = g_list_length(klinklist);
+  snprintf(kappbarmsg, 100, _("Kanji found: %d"), result_num);
   gnome_appbar_set_status(GNOME_APPBAR(kanjiDic->appbar_kanji), kappbarmsg);
 
-  if (count_kanji(klinklist) == 1) print_kanjinfo(klinklist->kanji);
+  if (result_num == 1) print_kanjinfo((gunichar) klinklist->data);
   
 
   // PRINT OUT KANJI FOUND
   node_ptr = klinklist;
   i = 0;
   while (node_ptr != NULL) { 
-    for (j = 0; j < 5; j++) kanji_result_str[j] = 0;
-    g_unichar_to_utf8(node_ptr->kanji, kanji_result_str);
+		kanji_list = g_list_prepend(kanji_list, node_ptr->data);
+		memset(kanji_result_str, 0, sizeof(kanji_result_str));
+    g_unichar_to_utf8((gunichar) node_ptr->data, kanji_result_str);
     //printf("%s\n", kanji_result_str);
     g_snprintf(kanji_result_labelstr, 100, "<span size=\"xx-large\">%s</span>", kanji_result_str); 
     kanji_results_anchor = gtk_text_buffer_create_child_anchor(kanjiDic->kanji_results_buffer, &kanjiDic->kanji_results_iter);
-
     
     if (gjitenApp->conf->bigkanji == TRUE) {
       kanji_result_label = gtk_label_new(NULL);
@@ -665,14 +594,39 @@ void on_kanji_search() {
     }
     gtk_widget_show(kanji_result_button);
     g_signal_connect_swapped(G_OBJECT(kanji_result_button), "clicked", G_CALLBACK(kanji_selected), 
-			     (gpointer) (node_ptr->kanji));
+														 node_ptr->data);
     gtk_text_view_add_child_at_anchor(GTK_TEXT_VIEW(kanjiDic->kanji_results_view), kanji_result_button, kanji_results_anchor);
-    node_ptr = node_ptr->next;
+    node_ptr = g_list_next(node_ptr);
   }
+
+	// find all different radicals in all the kanji found
+	if (GTK_IS_WIDGET(kanjiDic->window_radicals)) {
+		if (kanji_list != NULL) {
+			rad_info_hash = g_hash_table_new(NULL, NULL);
+			for (kanji_list_ptr = kanji_list;
+					 kanji_list_ptr != NULL;
+					 kanji_list_ptr = g_list_next(kanji_list_ptr)) {
+				kanji_info = g_hash_table_lookup(kanjiDic->kanji_info_hash, kanji_list_ptr->data);
+				for (rad_info_list = kanji_info->rad_info_list;
+						 rad_info_list != NULL;
+						 rad_info_list = g_list_next(rad_info_list)) {
+					g_hash_table_insert(rad_info_hash, (gpointer) ((RadInfo *) rad_info_list->data)->radical, rad_info_list->data);
+				}
+			}
+			g_hash_table_foreach(kanjiDic->rad_button_hash, (GHFunc) set_radical_button_unsensitive, (gpointer) FALSE);
+			
+			g_hash_table_foreach(rad_info_hash, (GHFunc) set_radical_button_sensitive, NULL);
+			g_hash_table_destroy(rad_info_hash);
+			g_list_free(kanji_list);
+		}
+		else { // none found, so set everything sensitive
+			g_hash_table_foreach(kanjiDic->rad_button_hash, (GHFunc) set_radical_button_unsensitive, (gpointer) TRUE);
+		}
+	}
 }
 
 
-int radical_selected(gunichar *radical) {
+int radical_selected(gunichar radical) {
   int i, j;
   gchar radical_selected[6];
   gchar tmpchar[6];
@@ -680,8 +634,8 @@ int radical_selected(gunichar *radical) {
   int removed;
   int radline_length = 0;
 
-  for (j = 0; j < 6; j++) radical_selected[j] = 0; 
-  g_unichar_to_utf8(*radical, radical_selected);
+  memset(radical_selected, 0, sizeof(radical_selected)); 
+  g_unichar_to_utf8(radical, radical_selected);
 
   radline_ptr = (gchar*) gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(kanjiDic->combo_entry_radical)->entry));
   newradline = g_strndup(radline_ptr, strlen(radline_ptr) + 6); //Enough space for one more character
@@ -691,7 +645,7 @@ int radical_selected(gunichar *radical) {
 
   removed = FALSE;
   for (i = 0; i < radline_length; i++) {  //Check if we already have the radical in the line
-    if (g_utf8_get_char(radline_ptr) != *radical) {
+    if (g_utf8_get_char(radline_ptr) != radical) {
       for (j = 0; j < 6; j++) tmpchar[j] = 0; 
       g_unichar_to_utf8(g_utf8_get_char(radline_ptr), tmpchar);
       strncat(newradline, tmpchar, 5);
@@ -700,10 +654,11 @@ int radical_selected(gunichar *radical) {
     radline_ptr = g_utf8_next_char(radline_ptr);
   }
   
-  if (removed == FALSE) strncat(newradline, radical_selected, 5); //Add the radical to the line
+	if (removed == FALSE) strncat(newradline, radical_selected, 5); //Add the radical to the line
   gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(kanjiDic->combo_entry_radical)->entry), newradline);
 
   g_free(newradline);
+
   on_kanji_search();
   
   return 0;
@@ -760,7 +715,7 @@ void history_add(gunichar unicharkanji) {
 void kanji_selected(gunichar kanji) {
   print_kanjinfo(kanji);
   history_add(kanji);
-  //printf("KANJI_SELECTED\n");
+  GJITEN_DEBUG("KANJI_SELECTED\n");
 }
 
 static void radical_window_close() {
@@ -771,36 +726,27 @@ static void radical_window_close() {
 }
 
 
-int get_radical_index(gunichar radical) {
-  int rad_index;
-  
-  for (rad_index = 0; rad_index < total_radicals + 1; rad_index++) {
-    if (radicals[rad_index].radical == radical) {
-      return rad_index; //we found it
-    }
-  }
-  return -1; //didn't find it
-}
-
 //get all radicals that the kanji has
 void get_rad_of_kanji(gunichar kanji) {
-  int rad_index, kanji_index, i;
   gchar *kdicline_ptr;
-  
+	KanjiInfo *kanji_info;
+	GList *rad_info_list;
+
   kdicline_ptr = kdic_line + RADICAL * KBUFSIZE;
-  for (rad_index = 0; rad_index < total_radicals; rad_index++) {
-    for (kanji_index = radical_kanji_start[rad_index]; 
-	 kanji_index < radical_kanji_start[rad_index] + radical_kanji_count[rad_index]; 
-	 kanji_index++) {
-      if (kanji == radical_kanji[kanji_index]) {
-	for (i = 0; i < 6; i++) kdicline_ptr[i] = 0;
-	g_unichar_to_utf8(radicals[rad_index].radical, kdicline_ptr);
-	kdicline_ptr = g_utf8_next_char(kdicline_ptr);
-	g_unichar_to_utf8(' ', kdicline_ptr);
-	kdicline_ptr = g_utf8_next_char(kdicline_ptr);
-      }
-    }
-  }
+
+	kanji_info = g_hash_table_lookup(kanjiDic->kanji_info_hash, (gconstpointer) kanji);
+	if (kanji_info != NULL) {
+		for (rad_info_list = kanji_info->rad_info_list;
+				 rad_info_list != NULL;
+				 rad_info_list = g_list_next(rad_info_list)) {
+			memset(kdicline_ptr, 0, 6);
+			if (kdicline_ptr >= kdic_line + RADICAL * KBUFSIZE + KBUFSIZE - 7) return;
+			g_unichar_to_utf8(((RadInfo *) rad_info_list->data)->radical, kdicline_ptr);
+			kdicline_ptr = g_utf8_next_char(kdicline_ptr);
+			g_unichar_to_utf8(' ', kdicline_ptr);
+			kdicline_ptr = g_utf8_next_char(kdicline_ptr);
+		}
+	}
 }
 
 static gunichar jis_radical_to_unicode(gchar *radical) {
@@ -823,15 +769,17 @@ static gunichar jis_radical_to_unicode(gchar *radical) {
 
 // Load the radical data from the file
 void load_radkfile() {
-  int rad_index, rad_max, kanji_index;
   int error = FALSE;
   struct stat radk_stat;
-  
+  gint rad_cnt = 0;
   gchar *radkfile_name = RADKFILE_NAME;
   gchar *radkfile_ptr;
   gchar *radkfile_end;
   int fd = 0;
-  
+  RadInfo *rad_info = NULL;
+	KanjiInfo *kanji_info;
+	gunichar kanji;
+
   if (radkfile != NULL) {
     //printf("radkfile already initialized.\n");
     return;
@@ -846,12 +794,9 @@ void load_radkfile() {
     g_error("**ERROR** radkfile: open()\n");
     error = TRUE;
   }
-  // printf("SIZE: %d\n", radkfile_size);
   radkfile = (gchar *) mmap(NULL, radkfile_size, PROT_READ, MAP_SHARED, fd, 0);
   if (radkfile == NULL) gjiten_abort_with_msg("mmap() failed for radkfile\n");
 
-  //  printf("STRLEN: %d\n", strlen(radkfile));
-
   if (error == TRUE) {
 		gjiten_print_error(_("Error opening %s.\n "\
 												 "Check your preferences or read the documentation!"),
@@ -859,40 +804,39 @@ void load_radkfile() {
 		return;
   }
 
-  radical_kanji = g_malloc(radkfile_size * sizeof(gunichar));
-  if (radical_kanji==NULL) gjiten_abort_with_msg("malloc failed for radical tables!\n");
-  
-  radkfile_end = radkfile + strlen(radkfile);
+  radkfile_end = radkfile + strlen(radkfile); //FIXME: lseek
   radkfile_ptr = radkfile;
-
-  rad_index = -1;
-  kanji_index = 0;
-  rad_max = 0;
   
+	if (kanjiDic->kanji_info_hash == NULL) {
+		kanjiDic->kanji_info_hash = g_hash_table_new(NULL, NULL);
+	}
+	if (kanjiDic->rad_info_hash == NULL) {
+		kanjiDic->rad_info_hash = g_hash_table_new(NULL, NULL);
+	}
+
   while((radkfile_ptr < radkfile_end) && (radkfile_ptr != NULL)) {
     if (*radkfile_ptr == '#') {  //find $ as first char on the line
       radkfile_ptr = get_eof_line(radkfile_ptr, radkfile_end); //Goto next line
       continue;
     }
     if (*radkfile_ptr == '$') {  //Radical info line
-      rad_index++;                         //Increase number of radicals found
+      rad_cnt++;          //Increase number of radicals found
       radkfile_ptr = g_utf8_next_char(radkfile_ptr);
-      while (g_unichar_iswide(g_utf8_get_char(radkfile_ptr)) == FALSE) //Find radical
+      while (g_unichar_iswide(g_utf8_get_char(radkfile_ptr)) == FALSE) { //Find radical
 				radkfile_ptr = g_utf8_next_char(radkfile_ptr);
+			}
 
-      radicals[rad_index].radical = jis_radical_to_unicode(radkfile_ptr); //store radical
+			rad_info = g_new0(RadInfo, 1);
+			rad_info->radical = jis_radical_to_unicode(radkfile_ptr); //store radical
 
-      while (g_ascii_isdigit(*radkfile_ptr) == FALSE) //Find stroke number
+      while (g_ascii_isdigit(*radkfile_ptr) == FALSE) { //Find stroke number
 				radkfile_ptr = g_utf8_next_char(radkfile_ptr);
+			}
 
-      radicals[rad_index].strokes = atoi(radkfile_ptr);  //Store the stroke number
-      radical_kanji_start[rad_index] = kanji_index;
-      radical_kanji_count[rad_index] = 0;
-      if (rad_index != 0) {
-				if (radical_kanji_count[rad_index - 1] > rad_max) 
-					rad_max = radical_kanji_count[rad_index - 1];  // find largest kanji count for a given radical
-      }
-      radkfile_ptr = get_eof_line(radkfile_ptr, radkfile_end); //Goto next line
+      rad_info->strokes = atoi(radkfile_ptr);  //Store the stroke number
+			kanjiDic->rad_info_list = g_list_append(kanjiDic->rad_info_list, rad_info);
+			g_hash_table_insert(kanjiDic->rad_info_hash, (gpointer) rad_info->radical, rad_info);
+			radkfile_ptr = get_eof_line(radkfile_ptr, radkfile_end); //Goto next line
     }
     else {   //Kanji
       while ((*radkfile_ptr != '$') && (radkfile_ptr < radkfile_end)) {
@@ -900,25 +844,31 @@ void load_radkfile() {
 					radkfile_ptr++;
 					continue;
 				}
-				//printf("KANJI_INDEX: %d \n", kanji_index);
-				radical_kanji[kanji_index] = g_utf8_get_char(radkfile_ptr); //Store kanji
+				kanji = g_utf8_get_char(radkfile_ptr);
+				kanji_info = g_hash_table_lookup(kanjiDic->kanji_info_hash, (gconstpointer) kanji);
+				if (kanji_info == NULL) {
+					kanji_info = g_new0(KanjiInfo, 1);
+					kanji_info->kanji = kanji;
+					g_hash_table_insert(kanjiDic->kanji_info_hash, (gpointer) kanji, (gpointer) kanji_info);
+				}
+				kanji_info->rad_info_list = g_list_prepend(kanji_info->rad_info_list, rad_info);
+				rad_info->kanji_info_list = g_list_prepend(rad_info->kanji_info_list, kanji_info);
 				radkfile_ptr = g_utf8_next_char(radkfile_ptr);
-				radical_kanji_count[rad_index]++;
-				kanji_index++; 
       }
     }
   }
-  total_radicals = rad_index;
 }
 
 static GtkWidget *create_window_radicals () {
-  int i = 0, j = 0, k = 0, rad_index = 0;
+  int i = 0, j = 0;
   int curr_strokecount = 0;
   GtkWidget *radtable; 
   GtkWidget *tmpwidget = NULL;
   GtkWidget *radical_label;
   gchar *strokenum_label;
   gchar radical[6];
+	RadInfo *rad_info = NULL;
+	GList *rad_info_list;
 
 	load_radkfile(); 
 
@@ -927,6 +877,13 @@ static GtkWidget *create_window_radicals () {
     gtk_widget_show(kanjiDic->window_radicals);
     return kanjiDic->window_radicals;
   }
+	if (kanjiDic->rad_button_hash != NULL)	{
+		g_hash_table_destroy(kanjiDic->rad_button_hash);
+		kanjiDic->rad_button_hash = NULL;
+	}
+	
+	kanjiDic->rad_button_hash = g_hash_table_new_full(NULL, NULL, NULL, NULL);
+	
   kanjiDic->window_radicals = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title(GTK_WINDOW(kanjiDic->window_radicals), _("Radicals"));
   g_signal_connect(GTK_OBJECT(kanjiDic->window_radicals), "destroy", GTK_SIGNAL_FUNC(radical_window_close), NULL);
@@ -935,27 +892,31 @@ static GtkWidget *create_window_radicals () {
   gtk_container_add(GTK_CONTAINER(kanjiDic->window_radicals), radtable);
   gtk_widget_show(radtable);
 
-   
-  for (rad_index = 0; rad_index <= total_radicals; rad_index++) {
+	for (rad_info_list = kanjiDic->rad_info_list; rad_info_list != NULL; rad_info_list = g_list_next(rad_info_list)) {
     if (i == RADLISTLEN) {
       i = 0;
       j++;
     }
-    if (curr_strokecount != radicals[rad_index].strokes) {
-      curr_strokecount = radicals[rad_index].strokes;
+		rad_info = (RadInfo *) rad_info_list->data;
+    if (curr_strokecount != rad_info->strokes) {
+			if (i == RADLISTLEN - 1) {
+				i = 0;
+				j++;
+			}
+      curr_strokecount = rad_info->strokes;
       strokenum_label = g_strdup_printf("<b>%d</b>", curr_strokecount); //Make a label with the strokenumber
       tmpwidget = gtk_label_new(""); //radical stroke number label
       gtk_label_set_markup(GTK_LABEL(tmpwidget), strokenum_label);
 			g_free(strokenum_label);
 
-      gtk_table_attach(GTK_TABLE(radtable), tmpwidget , i, i+1, j, j+1,
-		       (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
-		       (GtkAttachOptions)(0), 0, 0);
+      gtk_table_attach(GTK_TABLE(radtable), tmpwidget, i, i+1, j, j+1,
+											 (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
+											 (GtkAttachOptions)(0), 0, 0);
       gtk_widget_show(tmpwidget);
-      i++;    
+      i++;
     }
-    for (k = 0; k < 6; k++) radical[k] = 0;
-    g_unichar_to_utf8(radicals[rad_index].radical, radical);
+    memset(radical, 0, sizeof(radical));
+    g_unichar_to_utf8(rad_info->radical, radical);
     radical_label = gtk_label_new(radical);
     gtk_widget_show(radical_label);
     tmpwidget = gtk_button_new();
@@ -963,15 +924,15 @@ static GtkWidget *create_window_radicals () {
     if ((gjitenApp->conf->normalfont != NULL) && gjitenApp->conf->normalfont_desc != NULL) {
       gtk_widget_modify_font(radical_label, gjitenApp->conf->normalfont_desc);
     }
-    
     g_signal_connect_swapped(GTK_OBJECT(tmpwidget), "clicked", GTK_SIGNAL_FUNC(radical_selected), 
-			      (GtkObject*)(&(radicals[rad_index].radical)));
+														 (gpointer)(rad_info->radical));
     
     gtk_table_attach(GTK_TABLE(radtable), tmpwidget , i, i+1, j, j+1,
 		     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
 		     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
     //      gtk_widget_set_usize(tmpwidget,20,20);
     gtk_widget_show(tmpwidget);
+		g_hash_table_insert(kanjiDic->rad_button_hash, (gpointer) rad_info->radical, tmpwidget);
     i++;
   }  
   gtk_widget_show(kanjiDic->window_radicals);
@@ -996,16 +957,16 @@ static void kanjidic_close() {
 }
 
 void shade_kanjidic_widgets() {
-  gtk_widget_set_sensitive(kanjiDic->spinb_strokenum,GTK_TOGGLE_BUTTON(kanjiDic->checkb_stroke)->active);
-  gtk_widget_set_sensitive(kanjiDic->spinb_plusmin,GTK_TOGGLE_BUTTON(kanjiDic->checkb_stroke)->active);
-  gtk_widget_set_sensitive(kanjiDic->label_plusmin,GTK_TOGGLE_BUTTON(kanjiDic->checkb_stroke)->active);
+  gtk_widget_set_sensitive(kanjiDic->spinb_strokenum, GTK_TOGGLE_BUTTON(kanjiDic->checkb_stroke)->active);
+  gtk_widget_set_sensitive(kanjiDic->spinb_plusmin, GTK_TOGGLE_BUTTON(kanjiDic->checkb_stroke)->active);
+  gtk_widget_set_sensitive(kanjiDic->label_plusmin, GTK_TOGGLE_BUTTON(kanjiDic->checkb_stroke)->active);
 
-  gtk_widget_set_sensitive(kanjiDic->button_clearrad,GTK_TOGGLE_BUTTON(kanjiDic->checkb_radical)->active);
-  gtk_widget_set_sensitive(kanjiDic->button_radtable,GTK_TOGGLE_BUTTON(kanjiDic->checkb_radical)->active);
-  gtk_widget_set_sensitive(kanjiDic->combo_entry_radical,GTK_TOGGLE_BUTTON(kanjiDic->checkb_radical)->active);
+  gtk_widget_set_sensitive(kanjiDic->button_clearrad, GTK_TOGGLE_BUTTON(kanjiDic->checkb_radical)->active);
+  gtk_widget_set_sensitive(kanjiDic->button_radtable, GTK_TOGGLE_BUTTON(kanjiDic->checkb_radical)->active);
+  gtk_widget_set_sensitive(kanjiDic->combo_entry_radical, GTK_TOGGLE_BUTTON(kanjiDic->checkb_radical)->active);
      
-  gtk_widget_set_sensitive(kanjiDic->button_cleark,GTK_TOGGLE_BUTTON(kanjiDic->checkb_ksearch)->active);
-  gtk_widget_set_sensitive(kanjiDic->combo_entry_key,GTK_TOGGLE_BUTTON(kanjiDic->checkb_ksearch)->active);
+  gtk_widget_set_sensitive(kanjiDic->button_cleark, GTK_TOGGLE_BUTTON(kanjiDic->checkb_ksearch)->active);
+  gtk_widget_set_sensitive(kanjiDic->combo_entry_key, GTK_TOGGLE_BUTTON(kanjiDic->checkb_ksearch)->active);
 }
 
 
@@ -1080,9 +1041,16 @@ void kanjidic_apply_fonts() {
     gtk_widget_modify_font(GTK_COMBO(kanjiDic->combo_entry_key)->entry, gjitenApp->conf->normalfont_desc);
     gtk_widget_modify_font(GTK_COMBO(kanjiDic->combo_entry_radical)->entry, gjitenApp->conf->normalfont_desc);
   }
-
 }
 
+void clear_radical_entry_box(gpointer entrybox) {
+  gtk_entry_set_text(GTK_ENTRY(entrybox), "");
+	if (GTK_IS_WIDGET(kanjiDic->window_radicals)) {
+		g_hash_table_foreach(kanjiDic->rad_button_hash, (GHFunc) set_radical_button_unsensitive, (gpointer) TRUE);
+	}
+}
+
+
 KanjiDic *kanjidic_create() {
   GtkWidget *vbox_maink;
   GtkWidget *hbox_spinb;
@@ -1105,8 +1073,6 @@ KanjiDic *kanjidic_create() {
   GtkWidget *tmpimage;
 	GtkWidget *vpane;
 
-  load_radkfile();
-
   if (kanjiDic != NULL) {
 		gtk_window_present(GTK_WINDOW(kanjiDic->window));
 		return kanjiDic;
@@ -1115,6 +1081,8 @@ KanjiDic *kanjidic_create() {
 	kanjiDic = g_new0(KanjiDic, 1);
 	gjitenApp->kanjidic = kanjiDic;
 
+  load_radkfile();
+
   if (kdic_line == NULL) kdic_line = (gchar *)g_malloc(KCFGNUM * KBUFSIZE);
   if (kdic_line == NULL) gjiten_abort_with_msg("Couldn't allocate memory\n");
 
@@ -1236,7 +1204,7 @@ KanjiDic *kanjidic_create() {
   gtk_table_attach(GTK_TABLE(table_koptions), kanjiDic->button_clearrad, 2, 3, 1, 2,
                     (GtkAttachOptions)(0), (GtkAttachOptions)(0), 5, 0);
   g_signal_connect_swapped(GTK_OBJECT(kanjiDic->button_clearrad), "clicked", 
-													 GTK_SIGNAL_FUNC(gjiten_clear_entry_box), 
+													 GTK_SIGNAL_FUNC(clear_radical_entry_box), 
 													 GTK_OBJECT(GTK_COMBO(kanjiDic->combo_entry_radical)->entry));
 
   kanjiDic->button_cleark = gtk_button_new_with_label(_("Clear"));
