@@ -1,9 +1,10 @@
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: t; c-basic-offset: 2 -*- */
+/* vi: set ts=2 sw=2: */
 /* pref.c
    
    GJITEN : A JAPANESE DICTIONARY FOR GNOME
   
-   Copyright (C) 1999-2003 Botond Botyanszki <boti at rocketmail dot com>
+   Copyright (C) 1999-2005 Botond Botyanszki <boti@rocketmail.com>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published  by
@@ -345,7 +346,9 @@ void preferences_response_cb(GtkDialog *dialog, gint response, gpointer user_dat
 
 	gjitenApp->conf->search_kata_on_hira = GTK_TOGGLE_BUTTON(GETWIDGET("checkbutton_search_kata_on_hira"))->active;	
 	gjitenApp->conf->search_hira_on_kata = GTK_TOGGLE_BUTTON(GETWIDGET("checkbutton_search_hira_on_kata"))->active;	
-
+	gjitenApp->conf->verb_deinflection = GTK_TOGGLE_BUTTON(GETWIDGET("checkbutton_verb_deinflection"))->active;	
+	gjitenApp->conf->unicode_radicals = GTK_TOGGLE_BUTTON(GETWIDGET("checkbutton_unicode_radicals"))->active;	
+      
   gjitenApp->conf->numofdics = 0;
 
 	dicutil_unload_dic();
@@ -453,6 +456,8 @@ void create_dialog_preferences() {
 
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(GETWIDGET("checkbutton_search_kata_on_hira")), gjitenApp->conf->search_kata_on_hira);	
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(GETWIDGET("checkbutton_search_hira_on_kata")), gjitenApp->conf->search_hira_on_kata);	
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(GETWIDGET("checkbutton_verb_deinflection")), gjitenApp->conf->verb_deinflection);	
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(GETWIDGET("checkbutton_unicode_radicals")), gjitenApp->conf->unicode_radicals);	
 	
 	chdir(GJITEN_DICDIR);
 	//	gnome_file_entry_set_default_path(GNOME_FILE_ENTRY(tmpwidget), GJITEN_DICDIR);
@@ -467,7 +472,7 @@ void create_dialog_preferences() {
 
   for (tcol = 0; (tcol * 14 + trow < KCFGNUM) || (tcol < 3); tcol++) {
     for (trow = 0; (tcol * 14 + trow < KCFGNUM) && (trow < 14); trow++) {
-      checkb_prefs[tcol * 14 + trow] = gtk_check_button_new_with_label(strginfo[tcol * 14 + trow]);
+      checkb_prefs[tcol * 14 + trow] = gtk_check_button_new_with_label(_(strginfo[tcol * 14 + trow]));
       gtk_widget_show(checkb_prefs[tcol * 14 + trow]);
       gtk_table_attach(GTK_TABLE(tmpwidget), checkb_prefs[tcol * 14 + trow], tcol, tcol + 1, trow, trow + 1,
 											 (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
